@@ -12,8 +12,12 @@ from datetime import datetime
 
 from concurrent.futures import ThreadPoolExecutor
 
-
-base_dir = os.path.join(os.path.dirname(__file__), '..', '..')
+# Get the absolute path of the current script
+current_script_path = os.path.abspath(__file__)
+base_dir = os.path.abspath(
+    os.path.join(current_script_path, '..', '..', '..', '..')
+)
+package_dir = base_dir + '/gmx_python_sdk/'
 print('base_dir', base_dir)
 
 logging.basicConfig(
@@ -30,7 +34,6 @@ def execute_call(call):
 
 
 def execute_threading(function_calls):
-
     with ThreadPoolExecutor() as executor:
         results = list(executor.map(execute_call, function_calls))
     return results
@@ -51,7 +54,7 @@ contract_map = {
         },
         "exchangerouter":
         {
-            "contract_address": "0x7C68C7866A64FA2160F78EEaE12217FFbf871fa8",
+            "contract_address": "0x69C527fC77291722b52649E45c838e41be8Bf5d5",
             "abi_path": "contracts/arbitrum/exchangerouter.json"
         },
         "depositvault":
@@ -71,13 +74,18 @@ contract_map = {
         },
         "syntheticsreader":
         {
-            "contract_address": "0xf60becbba223EEA9495Da3f606753867eC10d139",
+            "contract_address": "0x5Ca84c34a381434786738735265b9f3FD814b824",
             "abi_path": "contracts/arbitrum/syntheticsreader.json"
         },
         "syntheticsrouter":
         {
             "contract_address": "0x7452c558d45f8afC8c83dAe62C3f8A5BE19c71f6",
             "abi_path": "contracts/arbitrum/syntheticsrouter.json"
+        },
+        "glvreader":
+        {
+            "contract_address": "0xd4f522c4339Ae0A90a156bd716715547e44Bed65",
+            "abi_path": "contracts/arbitrum/glvreader.json"
         }
     },
     'avalanche':
@@ -94,7 +102,7 @@ contract_map = {
         },
         "exchangerouter":
         {
-            "contract_address": "0x79be2F4eC8A4143BaF963206cF133f3710856D0a",
+            "contract_address": "0x3BE24AED1a4CcaDebF2956e02C27a00726D4327d",
             "abi_path": "contracts/avalanche/exchangerouter.json"
         },
         "depositvault":
@@ -114,7 +122,7 @@ contract_map = {
         },
         "syntheticsreader":
         {
-            "contract_address": "0x1D5d64d691FBcD8C80A2FD6A9382dF0fe544cBd8",
+            "contract_address": "0xBAD04dDcc5CC284A86493aFA75D2BEb970C72216",
             "abi_path": "contracts/avalanche/syntheticsreader.json"
         },
         "syntheticsrouter":
@@ -126,96 +134,51 @@ contract_map = {
 }
 
 
-class Config:
+class ConfigManager:
 
-    def __init__(
-            self,
-            filepath: str = os.path.join(base_dir, "config.yaml")
-    ):
-        self.file_path = filepath
-        self.skeleton = {
-            'arbitrum': {
-                'rpc': None,
-                'chain_id': None
-            },
-            'avalanche': {
-                'rpc': None,
-                'chain_id': None
-            },
-            'private_key': None,
-            'user_wallet_address': None
-        }
+    def __init__(self, chain: str):
 
-    def load_config(self):
-        try:
-            config = yaml.safe_load(open(os.path.join(base_dir, "config.yaml")))
-            return self.test_config_format(config)
-        except FileNotFoundError:
-            print(f"Config file '{self.file_path}' not found.\nLoading blank template!")
-            return self.skeleton
+        self.chain = chain
+        self.rpc = None
+        self.chain_id = None
+        self.user_wallet_address = None
+        self.private_key = None
+        self.tg_bot_token = None
 
-    def set_config(self, config):
-        print(f"Setting config file: '{self.file_path}'")
-        with open(self.file_path, 'w') as file:
-            yaml.dump(config, file)
+    def set_config(self, filepath: str = os.path.join(base_dir, "config.yaml")):
 
-    def test_config_format(self, config):
+        with open(filepath, 'r') as file:
+            config_file = yaml.safe_load(file)
 
-        if config.keys() == self.skeleton.keys():
-            return config
-        else:
-            structure = """
-                {
-                'arbitrum': {
-                    'rpc': None,
-                    'chain_id': None
-                    },
-                'avalanche': {
-                    'rpc': None,
-                    'chain_id': None
-                    },
-                'private_key': None,
-                'user_wallet_address': None
-                }"""
-            raise Exception(
-            "Please make sure your config file matches the following structure:\n\n{}".format(
-                structure
-                )
-            )
+        self.set_rpc(config_file['rpcs'][self.chain])
+        self.set_chain_id(config_file['chain_ids'][self.chain])
+        self.set_wallet_address(config_file['user_wallet_address'])
+        self.set_private_key(config_file['private_key'])
+
+    def set_rpc(self, value):
+        self.rpc = value
+
+    def set_chain_id(self, value):
+        self.chain_id = value
+
+    def set_wallet_address(self, value):
+        self.user_wallet_address = value
+
+    def set_private_key(self, value):
+        self.private_key = value
 
 
-def get_config(filepath: str = os.path.join(base_dir, "config.yaml")):
-
-    config = Config(filepath).load_config()
-
-    if config['private_key'] is None:
-        logging.warning("Private key not set!")
-
-    if config['arbitrum']['rpc'] is None:
-        logging.warning("Arbitrum RPC not set!")
-
-    if config['avalanche']['rpc'] is None:
-        logging.warning("Avalanche RPC not set!")
-
-    if config['user_wallet_address'] is None:
-        logging.warning("Wallet address not set!")
-
-    return config
-
-
-def create_connection(rpc: str = None, chain: str = None):
+def create_connection(config):
     """
     Create a connection to the blockchain
     """
-    if rpc is None:
-        rpc = get_config()[chain]['rpc']
 
-    web3_obj = Web3(Web3.HTTPProvider(rpc))
+    web3_obj = Web3(Web3.HTTPProvider(config.rpc))
 
     return web3_obj
 
 
-def convert_to_checksum_address(chain: str, address: str):
+def convert_to_checksum_address(config, address: str):
     """
     Convert a given address to checksum format
 
@@ -232,8 +195,7 @@ def convert_to_checksum_address(chain: str, address: str):
         checksum formatted address.
 
     """
-
-    web3_obj = create_connection(chain=chain)
+    web3_obj = create_connection(config)
 
     # Added to support older versions of web3.py for now
     try:
@@ -268,6 +230,7 @@ def get_contract_object(web3_obj, contract_name: str, chain: str):
         open(
             os.path.join(
                 base_dir,
+                'gmx_python_sdk',
                 contract_map[chain][contract_name]["abi_path"]
             )
         )
@@ -278,7 +241,7 @@ def get_contract_object(web3_obj, contract_name: str, chain: str):
     )
 
 
-def get_token_balance_contract(chain: str, contract_address: str):
+def get_token_balance_contract(config: str, contract_address: str):
     """
     Get the contract object required to query a users token balance
 
@@ -290,13 +253,12 @@ def get_token_balance_contract(chain: str, contract_address: str):
         the token to determine the balance of.
 
     """
-    rpc = get_config()[chain]['rpc']
 
-    web3_obj = create_connection(rpc)
+    web3_obj = create_connection(config)
     contract_abi = json.load(
         open(
             os.path.join(
-                base_dir,
+                package_dir,
                 'contracts',
                 'balance_abi.json'
             )
@@ -323,7 +285,6 @@ def get_tokens_address_dict(chain: str):
         dictionary containing available tokens to trade on GMX.
 
     """
-
     url = {
         "arbitrum": "https://arbitrum-api.gmxinfra.io/tokens",
         "avalanche": "https://avalanche-api.gmxinfra.io/tokens"
@@ -350,7 +311,7 @@ def get_tokens_address_dict(chain: str):
     return token_address_dict
 
 
-def get_reader_contract(chain: str):
+def get_reader_contract(config):
     """
     Get a reader contract web3_obj for a given chain
 
@@ -360,17 +321,16 @@ def get_reader_contract(chain: str):
         avalanche or arbitrum.
 
     """
-    rpc = get_config()[chain]['rpc']
 
-    web3_obj = create_connection(rpc)
+    web3_obj = create_connection(config)
     return get_contract_object(
         web3_obj,
         'syntheticsreader',
-        chain
+        config.chain
     )
 
 
-def get_event_emitter_contract(chain: str):
+def get_event_emitter_contract(config):
     """
     Get a event emitter contract web3_obj for a given chain
 
@@ -380,17 +340,16 @@ def get_event_emitter_contract(chain: str):
         avalanche or arbitrum.
 
     """
-    rpc = get_config()[chain]['rpc']
 
-    web3_obj = create_connection(rpc)
+    web3_obj = create_connection(config)
     return get_contract_object(
         web3_obj,
         'eventemitter',
-        chain
+        config.chain
     )
 
 
-def get_datastore_contract(chain: str):
+def get_datastore_contract(config):
     """
     Get a datastore contract web3_obj for a given chain
 
@@ -400,17 +359,16 @@ def get_datastore_contract(chain: str):
         avalanche or arbitrum.
 
     """
-    rpc = get_config()[chain]['rpc']
 
-    web3_obj = create_connection(rpc)
+    web3_obj = create_connection(config)
     return get_contract_object(
         web3_obj,
         'datastore',
-        chain
+        config.chain
     )
 
 
-def get_exchange_router_contract(chain: str):
+def get_exchange_router_contract(config):
     """
     Get a exchange router contract web3_obj for a given chain
 
@@ -420,17 +378,35 @@ def get_exchange_router_contract(chain: str):
         avalanche or arbitrum.
 
     """
-    rpc = get_config()[chain]['rpc']
 
-    web3_obj = create_connection(rpc)
+    web3_obj = create_connection(config)
     return get_contract_object(
         web3_obj,
         'exchangerouter',
-        chain
+        config.chain
     )
 
 
-def create_signer(chain: str):
+def get_glv_reader_contract(config):
+    """
+    Get a glv reader contract web3_obj for a given chain
+
+    Parameters
+    ----------
+    chain : str
+        avalanche or arbitrum.
+
+    """
+
+    web3_obj = create_connection(config)
+    return get_contract_object(
+        web3_obj,
+        'glvreader',
+        config.chain
+    )
+
+
+def create_signer(config: str):
     """
     Creastea a signer for a given chain
 
@@ -440,10 +416,9 @@ def create_signer(chain: str):
         avalanche or arbitrum.
 
     """
-    config = get_config()
 
-    private_key = config['private_key']
-    rpc = config[chain]['rpc']
+    private_key = config.private_key
+    rpc = config.rpc
     web3_obj = create_connection(rpc)
 
     return web3_obj.eth.account.from_key(private_key)
@@ -489,7 +464,9 @@ def create_hash_string(string: str):
     return create_hash(["string"], [string])
 
 
-def get_execution_price_and_price_impact(chain: str, params: dict, decimals: int):
+def get_execution_price_and_price_impact(
+    config, params: dict, decimals: int
+):
     """
     Get the execution price and price impact for a position
 
@@ -503,8 +480,7 @@ def get_execution_price_and_price_impact(chain: str, params: dict, decimals: int
         number of decimals of the token being traded eg ETH == 18.
 
     """
-
-    reader_contract_obj = get_reader_contract(chain)
+    reader_contract_obj = get_reader_contract(config)
 
     output = reader_contract_obj.functions.getExecutionPrice(
         params['data_store_address'],
@@ -520,7 +496,7 @@ def get_execution_price_and_price_impact(chain: str, params: dict, decimals: int
             'price_impact_usd': output[0] / 10**PRECISION}
 
 
-def get_estimated_swap_output(chain: str, params: dict):
+def get_estimated_swap_output(config, params: dict):
     """
     For a given chain and requested swap get the amount of tokens
     out and the price impact the swap will have.
@@ -533,8 +509,7 @@ def get_estimated_swap_output(chain: str, params: dict):
         dictionary of the swap parameters.
 
     """
-
-    reader_contract_obj = get_reader_contract(chain)
+    reader_contract_obj = get_reader_contract(config)
 
     output = reader_contract_obj.functions.getSwapAmountOut(
         params['data_store_address'],
@@ -550,7 +525,7 @@ def get_estimated_swap_output(chain: str, params: dict):
             }
 
 
-def get_estimated_deposit_amount_out(chain: str, params: dict):
+def get_estimated_deposit_amount_out(config, params: dict):
     """
     For a given chain and requested deposit amount get the amount of
     gm expected to be output.
@@ -563,8 +538,7 @@ def get_estimated_deposit_amount_out(chain: str, params: dict):
         dictionary of the gm input parameters.
 
     """
-
-    reader_contract_obj = get_reader_contract(chain)
+    reader_contract_obj = get_reader_contract(config)
 
     output = reader_contract_obj.functions.getDepositAmountOut(
         params['data_store_address'],
@@ -578,7 +552,7 @@ def get_estimated_deposit_amount_out(chain: str, params: dict):
     return output
 
 
-def get_estimated_withdrawal_amount_out(chain: str, params: dict):
+def get_estimated_withdrawal_amount_out(config, params: dict):
     """
     For a given chain and requested withdrawal amount get the amount of
     long/shorts tokens expected to be output.
@@ -591,8 +565,7 @@ def get_estimated_withdrawal_amount_out(chain: str, params: dict):
         dictionary of the gm parameters.
 
     """
-
-    reader_contract_obj = get_reader_contract(chain)
+    reader_contract_obj = get_reader_contract(config)
 
     output = reader_contract_obj.functions.getWithdrawalAmountOut(
         params['data_store_address'],
@@ -650,12 +623,10 @@ def apply_factor(value, factor):
     return value * factor / 10**30
 
 
-def get_funding_factor_per_period(market_info: dict,
-                                  is_long: bool,
-                                  period_in_seconds: int,
-                                  long_interest_usd: int,
-                                  short_interest_usd: int
-                                  ):
+def get_funding_factor_per_period(
+    market_info: dict, is_long: bool, period_in_seconds: int,
+    long_interest_usd: int, short_interest_usd: int
+):
     """
     For a given market, calculate the funding factor for a given period
 
@@ -674,7 +645,9 @@ def get_funding_factor_per_period(market_info: dict,
 
     """
 
-    funding_factor_per_second = market_info['funding_factor_per_second'] * 10**-28
+    funding_factor_per_second = (
+        market_info['funding_factor_per_second'] * 10**-28
+    )
 
     long_pays_shorts = market_info['is_long_pays_short']
 
@@ -717,9 +690,8 @@ def save_json_file_to_datastore(filename: str, data: dict):
         dictionary of data.
 
     """
-
     filepath = os.path.join(
-        base_dir,
+        package_dir,
         'data_store',
         filename
     )
@@ -738,7 +710,6 @@ def make_timestamped_dataframe(data):
         dataframe to add timestamp column to.
 
     """
-
     dataframe = pd.DataFrame(data, index=[0])
     dataframe['timestamp'] = datetime.now()
 
@@ -759,7 +730,7 @@ def save_csv_to_datastore(filename: str, dataframe):
     """
 
     archive_filepath = os.path.join(
-        base_dir,
+        package_dir,
         "data_store",
         filename
     )
@@ -775,7 +746,7 @@ def save_csv_to_datastore(filename: str, dataframe):
 
     dataframe.to_csv(
         os.path.join(
-            base_dir,
+            package_dir,
             "data_store",
             filename
         ),
@@ -806,6 +777,12 @@ def determine_swap_route(markets: dict, in_token: str, out_token: str):
 
     """
 
+    if in_token == "0x2f2a2543B76A4166549F7aaB2e75Bef0aefC5B0f":
+        in_token = "0x47904963fc8b2340414262125aF798B9655E58Cd"
+
+    if out_token == "0x2f2a2543B76A4166549F7aaB2e75Bef0aefC5B0f":
+        out_token = "0x47904963fc8b2340414262125aF798B9655E58Cd"
+
     if in_token == "0xaf88d065e77c8cC2239327C5EDb3A432268e5831":
         gmx_market_address = find_dictionary_by_key_value(
             markets,
@@ -813,8 +790,6 @@ def determine_swap_route(markets: dict, in_token: str, out_token: str):
             out_token
         )['gmx_market_address']
     else:
-        if in_token == "0x2f2a2543B76A4166549F7aaB2e75Bef0aefC5B0f":
-            in_token = "0x47904963fc8b2340414262125aF798B9655E58Cd"
         gmx_market_address = find_dictionary_by_key_value(
             markets,
             "index_token_address",
@@ -826,8 +801,6 @@ def determine_swap_route(markets: dict, in_token: str, out_token: str):
     if out_token != "0xaf88d065e77c8cC2239327C5EDb3A432268e5831" and \
             in_token != "0xaf88d065e77c8cC2239327C5EDb3A432268e5831":
         is_requires_multi_swap = True
-        if out_token == "0x2f2a2543B76A4166549F7aaB2e75Bef0aefC5B0f":
-            out_token = "0x47904963fc8b2340414262125aF798B9655E58Cd"
         second_gmx_market_address = find_dictionary_by_key_value(
             markets,
             "index_token_address",
@@ -840,5 +813,5 @@ def determine_swap_route(markets: dict, in_token: str, out_token: str):
 
 
 if __name__ == "__main__":
-
-    output = get_tokens_address_dict('avalanche')
+    arbitrum_config_object = ConfigManager(chain='arbitrum')
+    arbitrum_config_object.set_config()
